function [X_opt, Y_opt, execTime] = moce(f, l, u, options, g, gamma)
%MOCE Simple Multi-Objective Cross-Entropy optimization heuristic
%   Applies the cross-entropy method to a multi-objective optimization 
%   problem
%
%   Syntax:
%      [X_opt, Y_opt, execTime] = MOCE(f, l, u, options, g, gamma)
%
%   Arguments:
%      f [function reference or inline function] Objective functions.
%      l [real, n-values vector] Lower bound of the decision variables.
%      U [real, n-values vector] Upper bound of the decision variables.
%      options [structure] Optimization options. Can be defined by using
%         the function CEOPTDEF.
%      g [function reference or inline function]. Constraints functions. It
%         must return a p-values vector. For unconstrained problems this
%         argument should be empty.
%      gamma [real, p-values vector] Constraints penalty factors. For
%         unconstrained problems this argument should be empty.
%
%   Outputs:
%      X_opt [q-by-n matrix] Obtained Pareto set.
%      Y_opt [q-by-m matrix] Obtained Pareto front.
%      execTime [real, sacalar] Execution time
%
%   Ref: Haber, R.E.; Beruvides, G.; Quiza, R. ; Hernndez, A., 2017. "A
%        simple multi-objective optimization based on the cross-entropy
%        method. IEEE Access, 5 (1), pp. 22272-22281, DOI: 10.1109/
%        ACCESS.2017.2764047.
% 
%   Cross-Entropy Optimisation Toolbox
%   (c) 2018, GAMHE <http://www.gamhe.eu> and CEFAS <http://cefas.umcc.cu>
%
   % Verifying the arguments values (F)
   if (nargin < 1), error('Not enough input arguments.'); end
   if isempty(f), error('Argument ''f'' should not be empty.'); end
   if ~(isa(f, 'function_handle') || isa(f, 'inline')), error('Argument ''f'' should be a function reference or an inline funtion.'); end
   % L
   if (nargin < 2), error('Not enough input arguments.'); end
   if isempty(l), error('Argument ''l'' should not be empty.'); end
   if ~isnumeric(l), error('Argument ''l'' should be a numeric value.'); end
   if (size(l, 1) > 1), error('Argument ''l'' should be a single row vector.'); end
   % U
   if (nargin < 3), error('Not enough input arguments.'); end
   if isempty(u), error('Argument ''u'' should not be empty.'); end
   if ~isnumeric(u), error('Argument ''u'' should be a numeric value.'); end
   if (size(u, 1) > 1), error('Argument ''u'' should be a single row vector.'); end
   if ~(all(size(l) == size(u))), error('Arguments ''l'' and ''u'' should be vectors with the same length.'); end
   if any(u <= l), error('All values of argument ''u'' must be higher than the corresponding values of ''l''.'); end
   % OPTIONS
   if (nargin < 4), options = ceoptimset(); end
   if isempty(options), error('Argument ''options'' should not be empty.'); end
   if ~isstruct(options), error('Argument ''options'' should be a structure.'); end
   includedFields = { 'Generations', 'PopulationSize', 'EliteRatio', 'HistogramIntervals', 'Display' };
   for i = 1 : length(includedFields)
      if ~isfield(options, includedFields{i}), error(['Argument ''options'' should contain a field called ''', includedFields{i}, '''.']); end
   end
   % G
   if (nargin < 5), g = []; end
   if ~(isempty(g) || isa(g, 'function_handle') || isa(g, 'inline')), error('Argument ''g'' should be a function reference or an inline funtion.'); end
   % GAMMA
   if (nargin < 6), gamma = []; end
   if (isempty(gamma) && ~isempty(g)), error('Argument ''gamma'' should not be empty if argument G is not.'); end
   if ~isempty(g)
      if ~isnumeric(gamma), error('Argument ''gamma'' should be a numeric value.'); end
      if (size(g, 1) > 1), error('Argument ''gamma'' should be a single row vector.'); end
   end

   % Algorithm parameters
   N = options.Generations;
   Z = options.PopulationSize;
   D = options.HistogramIntervals;
   alpha = options.EliteRatio;

   % Initializations
   n = length(u);
   m = length(f(l));
   E = [];
   if (~isempty(gamma))
       p = length(gamma);
   else
       p = 0;
   end

   % Evolution process
   if (strcmpi(options.Display, 'iter'))
      fprintf('   Generation      Time [s]  PF-size\n');
   end
   initTime = now;
   for t = 1 : N
      if isempty(E)
         % Creating initial random population
         Q(:, 1 : n) = repmat(l, Z, 1) + rand(Z, n).*repmat(u - l, Z, 1);
      else
         % Creating a new population from elite population
         b_min = min(E(:, n + 1 : n + m));
         b_max = max(E(:, n + 1 : n + m));
         %B = repmat(b_min, D + 1, 1) + repmat(b_max - b_min, D + 1, 1).* ...
         %                              repmat([0 : D].'./D, 1, m);
         for i = 1 : size(E, 1)
            b = floor((D - 1).*(E(i, n + 1 : n + m) - b_min)./(b_max - b_min));
            e(i, 1) = sum(b.*D.^[m - 1 : -1 : 0]) + 1;
         end
         Q = [];
         for i = 1 : D^m
            id_e = find(e == i);
            if ~isempty(id_e)
               mu = mean(E(id_e, 1 : n), 1);
               sigma = (u - l)./D;
               N_e = round(length(id_e).*(Z - floor(alpha.*Z))./floor(alpha.*Z));
               Q = [Q; randnt(N_e, n, mu, sigma, l, u)]; 
            end
         end
         Q = [Q; E(:, 1 : n)];
      end
      % Evaluating population
      Q(:, n + 1 : n + m) = f(Q(:, 1 : n));
      if (p > 0)
          G = sum(repmat(gamma, size(Q, 1), 1).*max(0, g(Q(:, 1 : n))), 2);
          Q(:, n + 1 : n + m) = Q(:, n + 1 : n + m) + repmat(G, 1, m);
      end
      % Evaluating dominance
      Q(:, n + m + 1) = prank(Q(:, n + 1 : n + m));
      % Extracting elite population
      Q = sortrows(Q, n + m + 1);
      E = Q(1 : round(alpha.*Z), :);
      if (strcmpi(options.Display, 'iter'))
          fprintf('%6d of %d %13.4f   %6d\n', t, N, ...
                  (now - initTime)*24*60*60, size(E, 1));
      end
   end
   % Filtering elite population
   E(:, end) = prank(E(:, n + 1 : n + m));
   E(find(E(:, end) > 0), :) = [];
   % Setting outcomes
   execTime = (now - initTime).*24.*60.*60;
   X_opt = E(:, 1 : n);
   Y_opt = E(:, n + 1 : n + m);
   % Showing process completion
   if (strcmpi(options.Display, 'iter') ||strcmpi(options.Display, 'final'))
         fprintf('Optimization process completed.\n');
         fprintf('   Epoch number: %d.\n', t);
         for i = 1 : m
             fprintf('   Objective function f%d(x) = [%f ... %f].\n', i, ...
                     min(Y_opt(:, i)),  max(Y_opt(:, i)));
         end
         for i = 1 : n
             fprintf('   Decision variable x%d = [%f ... %f].\n', i, ...
                     min(X_opt(:, i)),  max(X_opt(:, i)));
         end
   end 
end
