function HA = pfm_ha(PF, p0)
% PFM_HA Hyperarea
%   Compute the Pareto Front metric hyperarea for two-objectives problems.  
%
%   Syntax: hv = PFM_HA(PF, p0)
%
%   Arguments:
%     PF [real, n-by-m matrix] Pareto front of n objectives and m points
%     p0 [real, m-values row vector] Reference point
%
%   Returns:
%     ha [numeric scalar] Hyperarea value
%
%   Cross-Entropy Optimisation Toolbox
%   (c) 2018, GAMHE <http://www.gamhe.eu> and CEFAS <http://cefas.umcc.cu>
%

    % Testing inputs
    if (nargin < 1), error('Not enough input arguments.'); end
    if (nargin < 2), p0 = 1.1.*max(PF); end
    % PF (Pareto front)
    if isempty(PF), error('Argument ''PF'' should not be empty.'); end
    if ~isnumeric(PF), error('Argument ''PF'' should contain only numeric values.'); end
    if (size(PF, 2) ~= 2), error('Argument ''PF'' should contain 2 columns.'); end
    % p0 (Reference point)
    if isempty(p0), error('Argument ''p0'' should not be empty.'); end
    if ~isnumeric(p0), error('Argument ''p0'' should a numeric value.'); end   
    if (size(p0, 1) ~= 1), error('Argument ''p0'' should be a row vector.'); end
    if (size(p0, 2) ~= size(PF, 2)), error('Argument ''p0'' should have the same columns number than ''PF''.'); end

    % Removing points from PF which are outside the hypercuboid [0, p0]
    PF = sortrows(PF, 1);
    id = any(PF > repmat(p0, size(PF, 1), 1), 2);
    PF(id, :) = [];
    
    % Removing dominated points from PF
    q = prank(PF);
    PF(q > 0, :) = [];
    
    % Computing hyperarea value
    HA = 0;
    for i = 1 : size(PF, 1)
        HA = HA + prod(abs(PF(i, :) - p0));
        p0 = [p0(1), PF(i, 2)];
    end
end