function hr = pfm_hr(PF, TPF, p0)
% PFM_HR Hyperarea or hypervolume ratio
%   Compute the Pareto Front metric hyperarea or hypervolume ratio
%
%   Syntax: hr = PFM_HA(PF, TPF, p0)
%
%   Arguments:
%     PF [real, n-by-m matrix] Pareto front of n objectives and m points
%     TPF [real, n-by-p matrix] True Pareto front of n objectives and p points
%     p0 [real, m-values row vector] Reference point
%
%   Returns:
%     hr [numeric scalar] Hyperarea or hypervolume ratio value
%
%   Cross-Entropy Optimisation Toolbox
%   (c) 2018, GAMHE <http://www.gamhe.eu> and CEFAS <http://cefas.umcc.cu>
%

    % Testing inputs
    if (nargin < 1), error('Not enough input arguments.'); end
    if (nargin < 2), p0 = 1.1.*max(PF); end
    % PF (Pareto front)
    if isempty(PF), error('Argument ''PF'' should not be empty.'); end
    if ~isnumeric(PF), error('Argument ''PF'' should contain only numeric values.'); end
    if (size(PF, 2) < 2), error('Argument ''PF'' should contain 2 or more columns.'); end
    % TPF (True Pareto front)
    if isempty(TPF), error('Argument ''TPF'' should not be empty.'); end
    if ~isnumeric(TPF), error('Argument ''TPF'' should contain only numeric values.'); end
    if (size(PF, 2) ~= size(TPF, 2)), error('Arguments ''PF'' and ''TPF'' should contain the same number of columns.'); end
    % p0 (Reference point)
    if isempty(p0), error('Argument ''p0'' should not be empty.'); end
    if ~isnumeric(p0), error('Argument ''p0'' should a numeric value.'); end   
    if (size(p0, 1) ~= 1), error('Argument ''p0'' should be a row vector.'); end
    if (size(p0, 2) ~= size(PF, 2)), error('Argument ''p0'' should have the same columns number than ''PF''.'); end

    if (size(PF, 2) > 2)
        % Computing hypervolumes (3 or more objectives)
        hk = pfm_hv(PF, p0);
        ht = pfm_hv(TPF, p0);
    else
        % Computing hyperarea (2 objectives)
        hk = pfm_ha(PF, p0);
        ht = pfm_ha(TPF, p0);       
    end
    
    % Computing ratio
    hr = hk/ht;  
end