function hv = pfm_hv(PF, p0, c)
% PFM_HV Hypervolume
%   Compute the Pareto Front metric hypervolume, which is approximated for
%   more than two objectives problems, by using c uniformly distributed 
%   random points within the bounded hyper-cuboid.  
%
%   Syntax: hv = PFM_HV(PF, p0, n)
%
%   Arguments:
%     PF [real, n-by-m matrix] Pareto front of n objectives and m points
%     p0 [real, m-values row vector] Reference point
%     c [integer, scalar] Number of evaluation points
%
%   Returns:
%     hv [numeric scalar] Hypervolume value
%
%   Cross-Entropy Optimisation Toolbox
%   (c) 2018, GAMHE <http://www.gamhe.eu> and CEFAS <http://cefas.umcc.cu>
%
%   Based on function "hypervolume" v. 1.0 by Yi Cao at Cranfield 
%   University on 20 April 2008

    % Testing inputs
    if (nargin < 1), error('Not enough input arguments.'); end
    if (nargin < 2), p0 = 1.1.*max(PF); end
    if (nargin < 3), c = 100^size(PF, 2); end
    % PF (Pareto front)
    if isempty(PF), error('Argument ''PF'' should not be empty.'); end
    if ~isnumeric(PF), error('Argument ''PF'' should contain only numeric values.'); end
    if (size(PF, 2) < 3), error('Argument ''PF'' should contain 3 or more columns.'); end
    % p0 (Reference point)
    if isempty(p0), error('Argument ''p0'' should not be empty.'); end
    if ~isnumeric(p0), error('Argument ''p0'' should a numeric value.'); end   
    if (size(p0, 1) ~= 1), error('Argument ''p0'' should be a row vector.'); end
    if (size(p0, 2) ~= size(PF, 2)), error('Argument ''p0'' should have the same columns number than ''PF''.'); end
    % c (Number of evaluation points)
    if isempty(c), error('Argument ''c'' should not be empty.'); end
    if (~isnumeric(c)), error('Argument ''c'' should be numeric.'); end
    if (max(size(c)) > 1), error('Argument ''c''  should be a scalar.'); end
    if (abs(c - round(c)) >= eps), error('Argument ''c''  should be integer.'); end
    if (c <= 1), error('Argument ''c''  should be greater than one.'); end
    
    % Removing points from PF which are outside the hypercuboid [0, p0]
    id = any(PF > repmat(p0, size(PF, 1), 1), 2);
    PF(id, :) = [];
    
    % Removing dominated points from PF
    q = prank(PF);
    PF(q > 0, :) = [];
    
    % Computing the normalized hypercuboid
    PF = PF*diag(1./p0);
    testPoints = rand(c, size(PF, 2));
    isDominated = false(c, 1);
    lowerBound = min(PF);
    
    % Evaluation the points dominance
    isChecked = all(testPoints > repmat(lowerBound, c, 1), 2);
    for i = 1 : size(PF, 1)
        if any(isChecked)
            f = all(bsxfun(@gt, testPoints(isChecked, :), PF(i, :)),2);
            isDominated(isChecked) = f;
            isChecked(isChecked) = ~f;
        end
    end
    
    % Computing the estimated hypevolume
    hv = prod(p0)*sum(isDominated)/c;
end

